using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using iAnywhere.Data.UltraLite;

namespace FileTransfer
{
    
    /// <summary>
    /// The class implements ULFileTransferProgressListener so that a control on the
    /// form can handle file transfer download events.
    /// </summary>
    public partial class Form1 : Form, ULFileTransferProgressListener
    {
        string _appPath = System.IO.Path.GetDirectoryName(
            System.Reflection.Assembly.GetExecutingAssembly().GetName().CodeBase);
        private ULConnection _conn = new ULConnection();

        /// <summary>
        /// A connection object is created on first initialization of the form. 
        /// The connection is attempted to see if the database exists. The Query
        /// button is enabled if it does.  
        /// </summary>
        public Form1()
        {
            InitializeComponent();
            _conn.ConnectionString = @"dbf=" + _appPath + @"\filetransfer.udb";
            try
            {
                // This is a check to see if the database file exists. 
                _conn.Open();
                btnQuery.Enabled = true;
                _conn.Close();
            }
            catch( ULException uex)
            {
                switch (uex.NativeError)
                {
                    case ULSQLCode.SQLE_ULTRALITE_DATABASE_NOT_FOUND:
                        //nothing
                        break;
                    default:
                        MessageBox.Show(uex.Message);
                        break;
                }
            }
        }

        /// <summary>
        /// ULFileTransferProgressListener method. This method is called during the file transfer
        /// operation.
        /// </summary>
        /// <param name="data">Object with information about the state of the file transfer.</param>
        /// <returns>False to carry on downloading, true to stop.</returns>
        /// <remarks>This implementation is very simple. More sophisticated methods would
        /// respond differently to different flags.</remarks>
        public bool FileTransferProgressed(ULFileTransferProgressData data)
        {
            switch (data.Flags)
            {
                //case ULFileTransferProgressData.FLAG_IS_BLOCKING:
                default:
                    DisplayProgress(data.BytesReceived, data.FileSize);
                    return false;
            }
        }

        private void Form1_Load(object sender, EventArgs e)
        {
            lbUser.SelectedIndex = 0;
        }

        /// <summary>
        /// The file transfer code is here.
        /// File transfer settings are created, and then the file is downloaded.
        /// </summary>
        /// <param name="sender">Standard argument for a click event</param>
        /// <param name="e">Standard argument for a click event</param>
        private void btnFileTransfer_Click(object sender, EventArgs e)
        {
            try
            {
                // Set up the file transfer object and its properties
                ULFileTransfer ft = new ULFileTransfer();
                ft.FileName = "filetransfer.udb";
                ft.DestinationPath = _appPath;
                ft.Stream = ULStreamType.TCPIP;
                ft.ForceDownload = true;
                ft.StreamParms = null;
                ft.UserName = lbUser.SelectedItem.ToString();
                ft.Password = "pwd_" + ft.UserName;
                ft.Version = "FileTransfer";
                
                btnFileTransfer.Enabled = false;
                progressBar1.Visible = true;
                textBox1.Text = "";
                
                // Download the file, showing the progress bar
                if (ft.DownloadFile( this ))
                {
                    progressBar1.Visible = false;
                    textBox1.Text = "File downloaded";
                    btnQuery.Enabled = true;
                }
                else
                {
                    switch (ft.StreamErrorCode)
                    {
                        default:
                            progressBar1.Visible = false;
                            textBox1.Text = "E. " + ft.StreamErrorCode.ToString();
                            break;
                    }
                    textBox1.Text += "\r\nFile not downloaded";
                }
            }
            catch (ULException uex)
            {
                textBox1.Text = uex.Message;
            }
            catch (Exception ex)
            {
                textBox1.Text = ex.Message;
            }
            finally
            {
                btnFileTransfer.Enabled = true;
            }
        }

        /// <summary>
        /// Get the first item from the first row in the table. This holds
        /// the user name in this case.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnQuery_Click(object sender, EventArgs e)
        {
            try
            {
                _conn.Open();
                ULCommand cmd = _conn.CreateCommand();
                cmd.CommandText = "select first name from t1";
                String name = cmd.ExecuteScalar().ToString();
                textBox1.Text = "Name is " + name;
                _conn.Close();
            }
            catch (ULException uex)
            {
                textBox1.Text = uex.Message;
            }
        }

        /// <summary>
        /// Function to update the file transfer progress bar. 
        /// </summary>
        /// <param name="BytesReceived">Number of bytes downloaded so far.</param>
        /// <param name="FileSize">Total number of bytes in the file.</param>
        public void DisplayProgress(ulong BytesReceived, ulong FileSize)
        {
            if (progressBar1.Maximum != (int)FileSize)
            {
                progressBar1.Maximum = (int)FileSize;
            }
            progressBar1.Value = (int)BytesReceived;
            this.Refresh();
        }

        /// <summary>
        /// Exit the application.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void menuItem1_Click(object sender, EventArgs e)
        {
            Application.Exit();
 
        }
    }
}